using System;
using System.Text;
using System.Web.UI.HtmlControls;
using System.Web.UI.WebControls;
using System.Web.UI;

namespace ReportViewer11
{
    /// <summary>
    /// Provides a Visual WebGui callable wrapper for Microsoft.Reporting.WebForms.ReportViewer.
    /// </summary>
    [Serializable(), System.ComponentModel.ToolboxItem(true)]
	public partial class ReportViewer : Gizmox.WebGUI.Forms.Hosts.AspControlBoxBase
	{
		public ReportViewer()
		{
			InitializeComponent();
		}		
		
	    public event EventHandler LoadComplete = delegate { };

        public void OnRender(object sender, EventArgs e)
        {
            LoadComplete(sender, e);
        }

        // 
        /// <summary>
        /// Creates the host page form control. 
        /// Adding a ScriptManager as Required by ReportViewer 11.0.0.0.
        /// </summary>
        /// <returns></returns>
        protected override HtmlForm CreateHostPageForm()
        {
            // Get Form from base
            HtmlForm objForm = base.CreateHostPageForm();

            // Add the ScriptManager as Required by ReportViewer 11.0.0.0.
            System.Web.UI.ScriptManager ScriptMgr = new System.Web.UI.ScriptManager();
            ScriptMgr.AsyncPostBackTimeout = 0;
            //ScriptMgr.ScriptMode = System.Web.UI.ScriptMode.Inherit;
            objForm.Controls.Add(ScriptMgr);

            objForm.Style.Add(HtmlTextWriterStyle.Width, "100%");
            objForm.Style.Add(HtmlTextWriterStyle.Height, "100%");
            objForm.Style.Add(HtmlTextWriterStyle.BackgroundColor, "White");

            // Return the Form
            return objForm;
        }

        protected override HtmlHead CreateHostPageHead()
        {
            HtmlHead objHead = base.CreateHostPageHead();
            HtmlGenericControl objInterfacesScript = new HtmlGenericControl();
            HtmlGenericControl objScript = new HtmlGenericControl();

            //objScript.TagName = "script";
            //objScript.Attributes.Add("type", "text/javascript");
            //objScript.InnerHtml = this.getFixInterfaceScript();
            //objHead.Controls.Add(objScript);

            objScript = new HtmlGenericControl();
            objScript.TagName = "script";
            objScript.Attributes.Add("type", "text/javascript");
            objScript.Attributes.Add("src", "Resources.Gizmox.WebGUI.Forms.Skins.CommonSkin.Interfaces.js" + Gizmox.WebGUI.Forms.VWGContext.Current.Config.GetFeatureValue("Extension", ".wgx"));
            objHead.Controls.Add(objScript);
            return objHead;
        }

        protected override System.Web.UI.HtmlControls.HtmlControl CreateHostPageBody()
        {
            HtmlControl objBody = base.CreateHostPageBody();
            return objBody;
        }

        protected override void OnHostedPagePreRender(object sender, EventArgs e)
        {
            ((Microsoft.Reporting.WebForms.ReportViewer)this.HostedControl).Style.Add("overflow", "auto");
            string strClientId = ((Microsoft.Reporting.WebForms.ReportViewer)this.HostedControl).ClientID;
            string strScript = getIsLoadingScript();
            strScript = strScript.Replace("##reportViewer##", strClientId);
            strScript = strScript.Replace("##reportViewerVWGID##", ((Gizmox.WebGUI.Forms.RegisteredComponent)this).ID.ToString());
            ScriptManager.RegisterStartupScript(this.HostedControl, this.HostedControlType, strClientId, strScript, true);

            base.OnHostedPagePreRender(sender, e);

            //Fix issue 5460 - ReportViewer In FF 3.5.3 - does not resize to container size when Dock = Fill
           //HtmlHead objHeader = ((System.Web.UI.Page)sender).Header;
            //if (objHeader != null)
            //{
                //ReportViewerTableStyle objStyle = new ReportViewerTableStyle();
                //objHeader.StyleSheet.CreateStyleRule(objStyle, null, "table#" + this.HostedControl.ClientID);
            //}
        }

        protected override void FireEvent(Gizmox.WebGUI.Common.Interfaces.IEvent objEvent)
        {
            try
            {
                if (objEvent.Type == "RVLoaded")
                {
                    LoadComplete(this, new EventArgs());
                }
                else
                    base.FireEvent(objEvent);
            }
            catch { }
        }

        /// <summary>
        /// IE fix according to:
        /// http://answers.microsoft.com/en-us/ie/forum/ie9-windows_7/error-in-ie-9-when-using-reportviewer-no-such/ed0d79f2-a72a-4f6d-98a0-876371f79b26?msgId=625ece4c-af49-4e30-9007-5a8ddc112588
        /// </summary>
        /// <returns></returns>
        private string getFixInterfaceScript()
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("if (window.getComputedStyle != null) {");
            sb.AppendLine("     var orginalGetComputedStyle = window.getComputedStyle;");
            sb.AppendLine("     window.getComputedStyle = function(element, parm)");
            sb.AppendLine("     {");
            sb.AppendLine("         try");
            sb.AppendLine("             {");
            sb.AppendLine("                 return orginalGetComputedStyle(element, null);");
            sb.AppendLine("             }");
            sb.AppendLine("         catch(err)");
            sb.AppendLine("             {");
            sb.AppendLine("                 return orginalGetComputedStyle(document.getElementsByTagName('body')[0], null);");
            sb.AppendLine("             }");
            sb.AppendLine("     }");
            sb.AppendLine("}");
            return sb.ToString();
        }

        private string getIsLoadingScript()
        {
            StringBuilder SB = new StringBuilder();
            SB.AppendLine("Sys.Application.add_load(function () {");
            SB.AppendLine("    $find(\"##reportViewer##\").add_propertyChanged(viewerPropertyChanged);");
            SB.AppendLine("});");
            SB.AppendLine(" ");
            SB.AppendLine("function viewerPropertyChanged(sender, e) {");
            SB.AppendLine("    if (e.get_propertyName() == 'isLoading') {");
            SB.AppendLine("        if ($find(\"##reportViewer##\").get_isLoading()) {");
            SB.AppendLine("            // Do something when loading starts");
            SB.AppendLine("        }");
            SB.AppendLine("        else {");
            SB.AppendLine("            var objEvent = VWG.Events.CreateEvent(\"##reportViewerVWGID##\", \"RVLoaded\");");
            SB.AppendLine("            VWG.Events.RaiseEvents();");
            SB.AppendLine("        }");
            SB.AppendLine("    }");
            SB.AppendLine("};");
            return SB.ToString();
        }

        /// <summary>
        /// Gets or sets a value indicating whether the container enables the user to scroll to any controls placed outside of its visible boundaries.
        /// </summary>
        /// <value>
        /// true if the container enables auto-scrolling; otherwise, false. The default value is true.
        /// </value>
        /// 
        protected override bool IsFixedSize
        {
            get
            {
                return false;
            }
        }

        public override bool AutoScroll
        {
            get
            {
                return false;
            }
            set
            {
                base.AutoScroll = value;
            }
        }

        /// <summary>
        /// 
        /// </summary>
        private class ReportViewerTableStyle : Style
        {
            internal ReportViewerTableStyle()
            {
            }

            /// <summary>
            /// Adds the specified object's style properties to a <see cref="T:System.Web.UI.CssStyleCollection"/> object.
            /// </summary>
            /// <param name="attributes">The <see cref="T:System.Web.UI.CssStyleCollection"/> object to which to add the style properties.</param>
            /// <param name="urlResolver">A <see cref="T:System.Web.UI.IUrlResolutionService"/> -implemented object that contains the context information for the current location (URL).</param>
            protected override void FillStyleAttributes(CssStyleCollection attributes, IUrlResolutionService urlResolver)
            {
                base.FillStyleAttributes(attributes, urlResolver);

                attributes[HtmlTextWriterStyle.Display] = "table !important";
            }
        }
	}
}
